import java.awt.*;

/**
   This class that displays debug information, reconfigures itself depending on
   what type of data it is send

   @author David Winchurch.
   @version 10/03/2001e
**/

public class DebugWindow extends Dialog
{
   //Pointers to the crucial components in the emulator
   private char[] MEM;
   private GameBoyCPU CPU;
   private GameBoyVideo VIDEO;

   //An integer used to store the type of DebugWindow this is
   private int debugType;

   //The various Panels used to get the layout right
   private Panel northPanel, eastPanel, southPanel, westPanel, centerPanel;

   //arrays that store the opcode descriptions
   private String[] opCodeList, cbOpCodeList;

   //The various components required to display the registry information
   private TextField A=new TextField(), B=new TextField(),
                     C=new TextField(), D=new TextField(),
                     E=new TextField(), F=new TextField(),
                     H=new TextField(), L=new TextField(),
                     AF=new TextField(), BC=new TextField(),
                     DE=new TextField(), HL=new TextField(),
                     PC=new TextField(), SP=new TextField(),
                     currentInstruction=new TextField(),
                     firstByte=new TextField(),
                     secondByte=new TextField(),
                     cpuCycles=new TextField(),
                     currentLine=new TextField();

   private Checkbox ZF=new Checkbox("Z-Flag"), NF=new Checkbox("N-Flag"),
                    HF=new Checkbox("H-Flag"), CF=new Checkbox("C-Flag"), IME=new Checkbox("IME");

   private Panel flagPanel1, flagPanel2;

   //The various components required to display the System information
   private TextField SB=new TextField(), SC=new TextField(),
                     DIV=new TextField(), TIM=new TextField(),
                     TM=new TextField(), TC=new TextField(),
                     mode=new TextField(),LYComp=new TextField(), LCYLY=new TextField();

   private Checkbox LCY=new Checkbox("LYC=LC"), MD2=new Checkbox("Mode 10"),
                    MD1=new Checkbox("Mode 01"), MD0=new Checkbox("Mode 00"),
                    VBLNK=new Checkbox("V-Blank"), LCDC=new Checkbox("LCDC"),
                    TO=new Checkbox("Timer"), SIO=new Checkbox("Serial"), JP=new Checkbox("Joypad");

   //The various components required to display the video information
   private TextField LCD=new TextField(), winPoint=new TextField(),
                     WIN=new TextField(), tileArea=new TextField(),
                     bgPoint=new TextField(), OBJSIZE=new TextField(),
                     OBJ=new TextField(), BG=new TextField(),
                     scrollY=new TextField(), scrollX=new TextField(),
                     winY=new TextField(), winX=new TextField();

   private Panel palTitle, BGWinPal, spr0pal, spr1pal;

   //The various components required to display the rom information
   private TextField TITLE=new TextField(), CLRF=new TextField(),
                     LC=new TextField(), SGBF=new TextField(),
                     CartType=new TextField(), RomSize=new TextField(),
                     RamSize=new TextField(), Dest=new TextField();

   //The various components required to display the palette information
   private Label[][] backPal=new Label[4][8], forePal=new Label[4][8];

   //Unchangeable integers used to tell wht type of debug window this is
   public static final int REG_INFO=0, SYS_INFO=1, VID_INFO=2, ROM_INFO=3, PAL_INFO=4;

   //Unchangeable values for the flag bits
   private final char Z_FLAG = 0x80; //1000 0000
   private final char N_FLAG = 0x40; //0100 0000
   private final char H_FLAG = 0x20; //0010 0000
   private final char C_FLAG = 0x10; //0001 0000

   /**
      Registry information constructor. Creates a debug window with all the
      components nessesary to display importanat registry information

      @param f the parent frame
      @param title the string you want displaying on the title bar
      @param cpu pointer to the virtual cpu
      @param type the type of debyg window this is
   **/

   public DebugWindow(Frame f, String title, GameBoyCPU cpu, int type)
   {
      //calls the DialogBox constuctor to set the parent, title and modality
      super(f, title, false);

      //provides a global pointer to the relevant memory area
      CPU=cpu;
      debugType=type;

      //sets up the debug window
      setLayout(new BorderLayout());
      setBackground(Color.lightGray);
      setLocation(f.getLocation().x+335,f.getLocation().y-62);

      //Creates the opcode string arrays
      opCodeList=createOpCodeList();
      cbOpCodeList=createCBOpCodeList();

      //None of the components should be editable
      A.setEnabled(false); B.setEnabled(false);
      C.setEnabled(false); D.setEnabled(false);
      E.setEnabled(false); F.setEnabled(false);
      H.setEnabled(false); L.setEnabled(false);
      AF.setEnabled(false); BC.setEnabled(false);
      DE.setEnabled(false); HL.setEnabled(false);
      PC.setEnabled(false); SP.setEnabled(false);
      currentInstruction.setEnabled(false);
      firstByte.setEnabled(false); secondByte.setEnabled(false);
      cpuCycles.setEnabled(false); currentLine.setEnabled(false);
      ZF.setEnabled(false); NF.setEnabled(false);
      HF.setEnabled(false); CF.setEnabled(false);
      IME.setEnabled(false);

      //Sets up and adds all the components to the north panel
      northPanel=new Panel(new GridLayout(2,11,3,3));
      northPanel.add(new Label("A")); northPanel.add(new Label("B"));
      northPanel.add(new Label("C")); northPanel.add(new Label("D"));
      northPanel.add(new Label("E")); northPanel.add(new Label("H"));
      northPanel.add(new Label("L")); northPanel.add(new Label("AF"));
      northPanel.add(new Label("BC")); northPanel.add(new Label("DE"));
      northPanel.add(new Label("HL")); northPanel.add(new Label("SP"));
      northPanel.add(new Label("PC"));
      northPanel.add(A); northPanel.add(B);
      northPanel.add(C); northPanel.add(D);
      northPanel.add(E); northPanel.add(H);
      northPanel.add(L); northPanel.add(AF);
      northPanel.add(BC); northPanel.add(DE);
      northPanel.add(HL); northPanel.add(SP);
      northPanel.add(PC);

      //Sets up and adds all the components to the south panel
      southPanel=new Panel(new GridLayout(2,5,3,3));
      southPanel.add(new Label("Current Instruction"));
      southPanel.add(new Label("First Byte"));
      southPanel.add(new Label("Second Byte"));
      southPanel.add(new Label("CPU Cycles"));
      southPanel.add(new Label("Current Line"));
      southPanel.add(currentInstruction);
      southPanel.add(firstByte);
      southPanel.add(secondByte);
      southPanel.add(cpuCycles);
      southPanel.add(currentLine);

      //Sets up and adds all the components to the center panel
      centerPanel=new Panel(new GridLayout(1,4,3,3));
      centerPanel.add(ZF);
      centerPanel.add(HF);
      centerPanel.add(NF);
      centerPanel.add(CF);
      centerPanel.add(IME);

      //Adds the panels to their relevant locations
      add(northPanel, BorderLayout.NORTH);
      add(centerPanel, BorderLayout.CENTER);
      add(southPanel, BorderLayout.SOUTH);

      //resizes the window to its optimum size
      pack();
   }

   /**
      System/rom information constructor. Creates a debug window with all the
      components nessesary to display importanat System/rom information

      @param f the parent frame
      @param title the string you want displaying on the title bar
      @param mem pointer to the virtual memory
      @param type the type of debyg window this is
   **/

   public DebugWindow(Frame f, String title, char[] mem, int type)
   {
      //calls the DialogBox constuctor to set the parent, title and modality
      super(f, title, false);

      //provides a global pointer to the relevant memory area
      MEM=mem;
      debugType=type;

      //sets up the debug window
      setLayout(new BorderLayout());
      setBackground(Color.lightGray);

      if(type==SYS_INFO)
      {
         //sets up the debug window
         setLocation(f.getLocation().x+645,f.getLocation().y+95);

         //None of the components should be editable
         SB.setEnabled(false); SC.setEnabled(false);
         DIV.setEnabled(false); TIM.setEnabled(false);
         TM.setEnabled(false); TC.setEnabled(false);
         LCY.setEnabled(false); MD2.setEnabled(false);
         MD1.setEnabled(false); MD0.setEnabled(false);
         LCYLY.setEnabled(false); mode.setEnabled(false);
         LYComp.setEnabled(false); VBLNK.setEnabled(false);
         LCDC.setEnabled(false); TO.setEnabled(false);
         SIO.setEnabled(false); JP.setEnabled(false);

         //Sets up and adds all the components to the center panel
         centerPanel=new Panel(new GridLayout(15,2,3,3));
         centerPanel.add(new Label("Serial transfer data: ")); centerPanel.add(SB);
         centerPanel.add(new Label("Serial IO Control: ")); centerPanel.add(SC);
         centerPanel.add(new Label("DIV: ")); centerPanel.add(DIV);
         centerPanel.add(new Label("Timer Counter: ")); centerPanel.add(TIM);
         centerPanel.add(new Label("Timer Modulo: ")); centerPanel.add(TM);
         centerPanel.add(new Label("Timer Control: ")); centerPanel.add(TC);
         centerPanel.add(new Label("Enabled LCDC", Label.RIGHT));
         centerPanel.add(new Label("Interrupts"));
         centerPanel.add(LCY); centerPanel.add(MD2);
         centerPanel.add(MD1); centerPanel.add(MD0);
         centerPanel.add(new Label("LYC=Current Line:")); centerPanel.add(LCYLY);
         centerPanel.add(new Label("Mode")); centerPanel.add(mode);
         centerPanel.add(new Label("LY Compare: ")); centerPanel.add(LYComp);
         centerPanel.add(new Label("Enabled Interrupts: ")); centerPanel.add(VBLNK);
         centerPanel.add(LCDC); centerPanel.add(TO);
         centerPanel.add(SIO); centerPanel.add(JP);

         //Adds the panel to its relevant location
         add(centerPanel, BorderLayout.CENTER);
      }
      else if(type==ROM_INFO)
      {
         //sets up the debug window
         setLocation(f.getLocation().x+335,f.getLocation().y+95);

         //None of the components should be editable
         TITLE.setEnabled(false); CLRF.setEnabled(false);
         LC.setEnabled(false); SGBF.setEnabled(false);
         CartType.setEnabled(false); RomSize.setEnabled(false);
         RamSize.setEnabled(false); Dest.setEnabled(false);

         //Sets up and adds all the components to the center panel
         centerPanel=new Panel(new GridLayout(8,2,3,3));
         centerPanel.add(new Label("Rom Title: ")); centerPanel.add(TITLE);
         centerPanel.add(new Label("Color Functions: ")); centerPanel.add(CLRF);
         centerPanel.add(new Label("Licence Code: ")); centerPanel.add(LC);
         centerPanel.add(new Label("SGB functions: ")); centerPanel.add(SGBF);
         centerPanel.add(new Label("Cart Type: ")); centerPanel.add(CartType);
         centerPanel.add(new Label("Rom Size: ")); centerPanel.add(RomSize);
         centerPanel.add(new Label("Ram Size: ")); centerPanel.add(RamSize);
         centerPanel.add(new Label("Destination: "));centerPanel.add(Dest);

         //Adds the panel to its relevant location
         add(centerPanel, BorderLayout.CENTER);
      }

      //resizes the window to its optimum size
      pack();
   }

   /**
      Video/palette information constructor. Creates a debug window with all the
      components nessesary to display importanat Video/palette information

      @param f the parent frame
      @param title the string you want displaying on the title bar
      @param vid pointer to the video subsystems
      @param type the type of debyg window this is
   **/

   public DebugWindow(Frame f, String title, GameBoyVideo vid, int type)
   {
      //calls the DialogBox constuctor to set the parent, title and modality
      super(f, title, false);

      //provides a global pointer to the relevant memory area
      VIDEO=vid;
      debugType=type;

      //sets up the debug window
      setLayout(new BorderLayout());
      setBackground(Color.lightGray);

      if(type==VID_INFO)
      {
         //sets up the debug window
         setLocation(f.getLocation().x,f.getLocation().y+343);

         //None of the components should be editable
         LCD.setEnabled(false); winPoint.setEnabled(false);
         WIN.setEnabled(false); tileArea.setEnabled(false);
         bgPoint.setEnabled(false); OBJSIZE.setEnabled(false);
         OBJ.setEnabled(false); BG.setEnabled(false);
         scrollX.setEnabled(false); scrollY.setEnabled(false);
         winX.setEnabled(false); winY.setEnabled(false);

         //Sets up and adds all the components to the center panel
         centerPanel=new Panel(new GridLayout(12,2,3,3));
         centerPanel.add(new Label("LCD: ")); centerPanel.add(LCD);
         centerPanel.add(new Label("Win Pointers: ")); centerPanel.add(winPoint);
         centerPanel.add(new Label("Window: ")); centerPanel.add(WIN);
         centerPanel.add(new Label("Tile Data Area: ")); centerPanel.add(tileArea);
         centerPanel.add(new Label("BG Pointers: ")); centerPanel.add(bgPoint);
         centerPanel.add(new Label("Obj Dimensions: ")); centerPanel.add(OBJSIZE);
         centerPanel.add(new Label("Objects: ")); centerPanel.add(OBJ);
         centerPanel.add(new Label("BackGround: ")); centerPanel.add(BG);
         centerPanel.add(new Label("BG X-Scroll Factor: ")); centerPanel.add(scrollX);
         centerPanel.add(new Label("BG Y-Scroll Factor: ")); centerPanel.add(scrollY);
         centerPanel.add(new Label("Win X-Scroll Factor: ")); centerPanel.add(winX);
         centerPanel.add(new Label("Win Y-Scroll Factor: ")); centerPanel.add(winY);

         //Adds the panel to its relevant location
         add(centerPanel, BorderLayout.CENTER);
      }
      else if(type==PAL_INFO)
      {
         //sets up the debug window
         setLocation(f.getLocation().x+335,f.getLocation().y+343);

         //Sets up and adds all the components to the west panel
         westPanel=new Panel(new GridLayout(9,5,3,3));
         westPanel.add(new Label(" "));
         westPanel.add(new Label("0"));
         westPanel.add(new Label("1"));
         westPanel.add(new Label("2"));
         westPanel.add(new Label("3"));

         //Sets up and adds all the components to the east panel
         eastPanel=new Panel(new GridLayout(9,5,3,3));
         eastPanel.add(new Label(" "));
         eastPanel.add(new Label("0"));
         eastPanel.add(new Label("1"));
         eastPanel.add(new Label("2"));
         eastPanel.add(new Label("3"));

         //Creates and sets up the labels used for displaying the palettes
         for(int i=0; i<8; i++)
            for(int j=0; j<5; j++)
            {
               if(j==0)
               {
                  westPanel.add(new Label(Integer.toString(i)));
                  eastPanel.add(new Label(" "));
               }
               else
               {
                  backPal[j-1][i]=new Label();
                  forePal[j-1][i]=new Label();
                  westPanel.add(backPal[j-1][i]);
                  eastPanel.add(forePal[j-1][i]);
               }
            }

         //Sets up and adds all the components to the north panel
         northPanel=new Panel(new GridLayout(1,2,3,3));
         northPanel.add(new Label("Background Palette", Label.CENTER));
         northPanel.add(new Label("Foreground Palette", Label.CENTER));

         //Adds the panels to their relevant locations
         add(northPanel, BorderLayout.NORTH);
         add(westPanel, BorderLayout.WEST);
         add(eastPanel, BorderLayout.EAST);
      }

      //resizes the window to its optimum size
      pack();
   }

   /**
      Makes any changes nessesary to the information displayed on the debuywindow
   **/

   public void refresh()
   {
      if(debugType==SYS_INFO)
      {
         SB.setText(Integer.toHexString((int)(MEM[0xFF01])));
         SC.setText((MEM[0xFF02]&0x80)>0? "Transfering, ":"Stopped, " + ((MEM[0xFF02]&0x01)>0? "Ext Clock":"Int Clock"));
         DIV.setText(Integer.toString(MEM[0xFF04]));
         TIM.setText(Integer.toString(MEM[0xFF05]));
         TM.setText(Integer.toString(MEM[0xFF06]));
         TC.setText(((MEM[0xFF07]&0x04)>0? "On, ":"Off, ") + ((MEM[0xFF07]&0x02)>0? ((MEM[0xFF07]&0x01)>0? "16.384 KHz":"65.536 KHz") : ((MEM[0xFF07]&0x01)>0? "262.144 KHz":"4.096 KHz")));

         LCY.setState((MEM[0xFF41]&0x40)>0);
         MD2.setState((MEM[0xFF41]&0x20)>0);
         MD1.setState((MEM[0xFF41]&0x10)>0);
         MD0.setState((MEM[0xFF41]&0x08)>0);

         LCYLY.setText((MEM[0xFF41]&0x04)>0? "Yes":"No");
         mode.setText((MEM[0xFF41]&0x02)>0? ((MEM[0xFF41]&0x01)>0? "11":"10") : ((MEM[0xFF41]&0x01)>0? "01":"00"));

         LYComp.setText(Integer.toHexString(MEM[0xFF45]));
         VBLNK.setState((MEM[0xFFFF]&0x01)>0);
         LCDC.setState((MEM[0xFFFF]&0x02)>0);
         TO.setState((MEM[0xFFFF]&0x04)>0);
         SIO.setState((MEM[0xFFFF]&0x08)>0);
         JP.setState((MEM[0xFFFF]&0x10)>0);
      }

      else if(debugType==REG_INFO)
      {
         A.setText(Integer.toHexString(CPU.getA()));
         B.setText(Integer.toHexString(CPU.getB()));
         C.setText(Integer.toHexString(CPU.getC()));
         D.setText(Integer.toHexString(CPU.getD()));
         E.setText(Integer.toHexString(CPU.getE()));
         H.setText(Integer.toHexString(CPU.getH()));
         L.setText(Integer.toHexString(CPU.getL()));

         AF.setText(Integer.toHexString(CPU.getAF()));
         BC.setText(Integer.toHexString(CPU.getBC()));
         DE.setText(Integer.toHexString(CPU.getDE()));
         HL.setText(Integer.toHexString(CPU.getHL()));
         PC.setText(Integer.toHexString(CPU.getPC()));
         SP.setText(Integer.toHexString(CPU.getSP()));

         ZF.setState((CPU.getF()&Z_FLAG)>0);
         NF.setState((CPU.getF()&N_FLAG)>0);
         HF.setState((CPU.getF()&H_FLAG)>0);
         CF.setState((CPU.getF()&C_FLAG)>0);
         IME.setState(CPU.getIME());

         currentInstruction.setText(getOpCodeText(CPU.getMem(CPU.getPC())));
         firstByte.setText(Integer.toHexString(CPU.getMem(CPU.getPC()+1)));
         secondByte.setText(Integer.toHexString(CPU.getMem(CPU.getPC()+2)));
         cpuCycles.setText(Integer.toString(CPU.getCpuCycles()));
         currentLine.setText(CPU.getVideo().getCurrentLine());
      }

      else if(debugType==VID_INFO)
      {
         LCD.setText(VIDEO.getLCD());
         winPoint.setText(VIDEO.getWinPointers());
         WIN.setText(VIDEO.getWIN());
         tileArea.setText(VIDEO.getTileArea());
         bgPoint.setText(VIDEO.getBGPointers());
         OBJSIZE.setText(VIDEO.getOBJSIZE());
         OBJ.setText(VIDEO.getOBJ());
         BG.setText(VIDEO.getBG());

         scrollY.setText(VIDEO.getScrollY());
         scrollX.setText(VIDEO.getScrollX());
         winY.setText(VIDEO.getWinY());
         winX.setText(VIDEO.getWinX());
      }

      else if(debugType==ROM_INFO)
      {
         String temp="";
         for(int i=0; i<16; i++) if(MEM[0x134+i]!=0x00) temp+=(char)(MEM[0x134+i]);
         TITLE.setText(temp);
         CLRF.setText(MEM[0x143]==0x80? "Yes":"No");
         if(MEM[0x14B]==0x33){temp=""; for(int i=0; i>2; i++) temp+=(char)(MEM[0x145-i]);}
         else if(MEM[0x14B]==0x79) temp="Accolade";
         else if(MEM[0x14B]==0xA4) temp="Konami";
         else temp="No Licence Code";
         LC.setText(temp);
         SGBF.setText(MEM[0x146]==0x03? "Yes":"No");
         CartType.setText(getCartType(MEM[0x147]));
         RomSize.setText(getRomSize(MEM[0x148]));
         RamSize.setText(getRamSize(MEM[0x149]));
         Dest.setText(MEM[0x14A]==0x00? "Japanese":"Non-Japanese");
      }

      else if(debugType==PAL_INFO)
      {
         for(int i=0; i<8; i++)
            for(int j=0; j<4; j++)
            {
               backPal[j][i].setBackground(VIDEO.getBackColour(j,i));
               forePal[j][i].setBackground(VIDEO.getForeColour(j,i));
            }
      }
   }

   /**
      gets the opcode text

      @param opCode the byte corresponding to the opcode
      @return the opcode as a string
   **/

   public String getOpCodeText(int opCode)
   {
      if(CPU.getHalt())
         return "HALT";
      //checks to see if the opcode is a CB opcode and returns the relevant string if it is
      else
         return (opCode==0xCB? cbOpCodeList[CPU.getMem(CPU.getPC()+1)]:opCodeList[opCode]);
   }

   /**
      gets the cartridge type the rom originally came from

      @return the cartridge type the rom originally came from as a string
      @param location 0x147 in the virtual memory
   **/

   public String getCartType(char b)
   {
      String temp=new String("Unknown Type");

      switch(b)
      {
         case 0x00 : {temp="ROM ONLY"; break;}
         case 0x01 : {temp="ROM+MBC1"; break;}
         case 0x02 : {temp="ROM+MBC1+CART_RAM"; break;}
         case 0x03 : {temp="ROM+MBC1+CART_RAM+BATT"; break;}
         case 0x05 : {temp="ROM+MBC2"; break;}
         case 0x06 : {temp="ROM+MBC2+BATTERY"; break;}
         case 0x08 : {temp="ROM+CART_RAM"; break;}
         case 0x09 : {temp="ROM+CART_RAM+BATTERY"; break;}
         case 0x0B : {temp="ROM+MMM01"; break;}
         case 0x0C : {temp="ROM+MMM01+SRAM"; break;}
         case 0x0D : {temp="ROM+MMM01+SRAM+BATT"; break;}
         case 0x0F : {temp="ROM+MBC3+TIMER+BATT"; break;}
         case 0x10 : {temp="ROM+MBC3+TIMER+CART_RAM+BATT"; break;}
         case 0x11 : {temp="ROM+MBC3"; break;}
         case 0x12 : {temp="ROM+MBC3+CART_RAM"; break;}
         case 0x13 : {temp="ROM+MBC3+CART_RAM+BATT"; break;}
         case 0x19 : {temp="ROM+MBC5"; break;}
         case 0x1A : {temp="ROM+MBC5+CART_RAM"; break;}
         case 0x1B : {temp="ROM+MBC5+CART_RAM+BATT"; break;}
         case 0x1C : {temp="ROM+MBC5+RUMBLE"; break;}
         case 0x1D : {temp="ROM+MBC5+RUMBLE+SRAM"; break;}
         case 0x1E : {temp="ROM+MBC5+RUMBLE+SRAM+BATT"; break;}
         case 0x1F : {temp="Pocket Camera"; break;}
         case 0xFD : {temp="Bandai TAMA5"; break;}
         case 0xFE : {temp="Hudson HuC-3"; break;}
         case 0xFF : {temp="Hudson HuC-1"; break;}
      }
      return temp;
   }

   /**
      gets the rom size of the cartridge the rom came from

      @return the rom size of the cartridge the rom came from as a string
      @param location 0x148 in the virtual memory
   **/

   public String getRomSize(char b)
   {
      String temp=new String("Unknown Size");

      switch(b)
      {
         case 0x00 : {temp="256Kbit=32KByte=2 banks"; break;}
         case 0x01 : {temp="512Kbit=64KByte=4 banks"; break;}
         case 0x02 : {temp="1Mbit=128KByte=8 banks"; break;}
         case 0x03 : {temp="2Mbit=256KByte=16 banks"; break;}
         case 0x04 : {temp="4Mbit=512KByte=32 banks"; break;}
         case 0x05 : {temp="8Mbit=1MByte=64 banks"; break;}
         case 0x06 : {temp="16Mbit=2MByte=128 banks"; break;}
         case 0x07 : {temp="32Mbit=4MByte=256 banks"; break;}
         case 0x52 : {temp="9Mbit=1.1MByte=72 banks"; break;}
         case 0x53 : {temp="10Mbit=1.2MByte=80 banks"; break;}
         case 0x54 : {temp="12Mbit=1.5MByte=96 banks"; break;}
      }
      return temp;
   }

   /**
      gets the ram size for the rom

      @return the ram size for the rom as a string
      @param location 0x149 in the virtual memory
   **/

   public String getRamSize(char b)
   {
      String temp=new String("Unknown Size");

      switch(b)
      {
         case 0x00 : {temp="None"; break;}
         case 0x01 : {temp="16kBit=2kB=1 bank"; break;}
         case 0x02 : {temp="64kBit=8kB=1 bank"; break;}
         case 0x03 : {temp="256kBit=32kB=4 banks"; break;}
         case 0x04 : {temp="1MBit=128kB=16 banks"; break;}
      }
      return temp;
   }

   /**
      Returns an array of strings corresponding to normal opcodes

      @return the opcode list as a string
   **/

   public String[] createOpCodeList()
   {
      String[] temp=
      {
         "00 NOP",
         "01 bb aa LD BC,$aabb",
         "02 LD (BC),A",
         "03 INC BC",
         "04 INC B",
         "05 DEC B",
         "06 xx LD B,$xx",
         "07 RLCA",
         "08 bb aa LD ($aabb),SP",
         "09 ADD HL,BC",
         "0A LD A,(BC)",
         "0B DEC BC",
         "0C INC C",
         "0D DEC C",
         "0E xx LD C,$xx",
         "0F RRCA",

         "10 00 STOP",
         "11 bb aa LD DE,$aabb",
         "12 LD (DE),A",
         "13 INC DE",
         "14 INC D",
         "15 DEC D",
         "16 xx LD D,$xx",
         "17 RLA",
         "18 xx JR $xx",
         "19 ADD HL,DE",
         "1A LD A,(DE)",
         "1B DEC DE",
         "1C INC E",
         "1D DEC E",
         "1E xx LD E,$xx",
         "1F RRA",

         "20 xx JR NZ,$xx",
         "21 bb aa LD HL,$aabb",
         "22 LD (HLI),A",
         "23 INC HL",
         "24 INC H",
         "25 DEC H",
         "26 xx LD H,$xx",
         "27 DAA",
         "28 xx JR Z,$xx",
         "29 ADD HL,HL",
         "2A LD A,(HLI)",
         "2B DEC HL",
         "2C INC L",
         "2D DEC L",
         "2E xx LD L,$xx",
         "2F CPL",

         "30 xx JR NC,$xx",
         "31 bb aa LD SP,$aabb",
         "32 LD (HLD),A",
         "33 INC SP",
         "34 INC (HL)",
         "35 DEC (HL)",
         "36 xx LD (HL),$xx",
         "37 SCF",
         "38 xx JR C,$xx",
         "39 ADD HL,SP",
         "3A LD A,(HLD)",
         "3B DEC SP",
         "3C INC A",
         "3D DEC A",
         "3E xx LD A,$xx",
         "3F CCF",

         "40 LD B,B",
         "41 LD B,C",
         "42 LD B,D",
         "43 LD B,E",
         "44 LD B,H",
         "45 LD B,L",
         "46 LD B,(HL)",
         "47 LD B,A",
         "48 LD C,B",
         "49 LD C,C",
         "4A LD C,D",
         "4B LD C,E",
         "4C LD C,H",
         "4D LD C,L",
         "4E LD C,(HL)",
         "4F LD C,A",

         "50 LD D,B",
         "51 LD D,C",
         "52 LD D,D",
         "53 LD D,E",
         "54 LD D,H",
         "55 LD D,L",
         "56 LD D,(HL)",
         "57 LD D,A",
         "58 LD E,B",
         "59 LD E,C",
         "5A LD E,D",
         "5B LD E,E",
         "5C LD E,H",
         "5D LD E,L",
         "5E LD E,(HL)",
         "5F LD E,A",

         "60 LD H,B",
         "61 LD H,C",
         "62 LD H,D",
         "63 LD H,E",
         "64 LD H,H",
         "65 LD H,L",
         "66 LD H,(HL)",
         "67 LD H,A",
         "68 LD L,B",
         "69 LD L,C",
         "6A LD L,D",
         "6B LD L,E",
         "6C LD L,H",
         "6D LD L,L",
         "6E LD L,(HL)",
         "6F LD L,A",

         "70 LD (HL),B",
         "71 LD (HL),C",
         "72 LD (HL),D",
         "73 LD (HL),E",
         "74 LD (HL),H",
         "75 LD (HL),L",
         "76 HALT",
         "77 LD (HL),A",
         "78 LD A,B",
         "79 LD A,C",
         "7A LD A,D",
         "7B LD A,E",
         "7C LD A,H",
         "7D LD A,L",
         "7E LD A,(HL)",
         "7F LD A,A",

         "80 ADD A,B",
         "81 ADD A,C",
         "82 ADD A,D",
         "83 ADD A,E",
         "84 ADD A,H",
         "85 ADD A,L",
         "86 ADD A,(HL)",
         "87 ADD A,A",
         "88 ADC A,B",
         "89 ADC A,C",
         "8A ADC A,D",
         "8B ADC A,E",
         "8C ADC A,H",
         "8D ADC A,L",
         "8E ADC A,(HL)",
         "8F ADC A,A",

         "90 SUB B",
         "91 SUB C",
         "92 SUB D",
         "93 SUB E",
         "94 SUB H",
         "95 SUB L",
         "96 SUB (HL)",
         "97 SUB A",
         "98 SBC A,B",
         "99 SBC A,C",
         "9A SBC A,D",
         "9B SBC A,E",
         "9C SBC A,H",
         "9D SBC A,L",
         "9E SBC A,(HL)",
         "9F SBC A,A",

         "A0 AND B",
         "A1 AND C",
         "A2 AND D",
         "A3 AND E",
         "A4 AND H",
         "A5 AND L",
         "A6 AND (HL)",
         "A7 AND A",
         "A8 XOR B",
         "A9 XOR C",
         "AA XOR D",
         "AB XOR E",
         "AC XOR H",
         "AD XOR L",
         "AE XOR (HL)",
         "AF XOR A",

         "B0 OR B",
         "B1 OR C",
         "B2 OR D",
         "B3 OR E",
         "B4 OR H",
         "B5 OR L",
         "B6 OR (HL)",
         "B7 OR A",
         "B8 CP B",
         "B9 CP C",
         "BA CP D",
         "BB CP E",
         "BC CP H",
         "BD CP L",
         "BE CP (HL)",
         "BF CP A",

         "C0 RET NZ",
         "C1 POP BC",
         "C2 bb aa JP NZ,$aabb",
         "C3 bb aa JP $aabb",
         "C4 bb aa CALL NZ,$aabb",
         "C5 PUSH BC",
         "C6 xx ADD A,$xx",
         "C7 RST $00",
         "C8 RET Z",
         "C9 RET",
         "CA bb aa JP Z,$aabb",
         "CB CB OPCODE MODIFIER!!!!!!!!!!!!",
         "CC bb aa CALL Z,$aabb",
         "CD bb aa CALL $aabb",
         "CE xx ADC A,$xx",
         "CF RST $08",

         "D0 RET NC",
         "D1 POP DE",
         "D2 bb aa JP NC,$aabb",
         "D3 NOP",
         "D4 bb aa CALL NC,$aabb",
         "D5 PUSH DE",
         "D6 xx SUB $xx",
         "D7 RST $10",
         "D8 RET C",
         "D9 RETI",
         "DA bb aa JP C,$aabb",
         "DB NOP",
         "DC bb aa CALL C,$aabb",
         "DD NOP",
         "DE xx SBC A,$xx",
         "DF RST $18",

         "E0 xx LD ($FFxx),A",
         "E1 POP HL",
         "E2 LD ($FF00+C),A",
         "E3 NOP",
         "E4 NOP",
         "E5 PUSH HL",
         "E6 xx AND $xx",
         "E7 RST $20",
         "E8 xx ADD SP,xx",
         "E9 JP (HL)",
         "EA bb aa LD ($aabb),A",
         "EB NOP",
         "EC NOP",
         "ED NOP",
         "EE xx XOR $xx",
         "EF RST $28",

         "F0 xx LD A,($FFxx)",
         "F1 POP AF",
         "F2 LD A,($FF00+C)",
         "F3 DI",
         "F4 NOP",
         "F5 PUSH AF",
         "F6 xx OR $xx",
         "F7 RST $30",
         "F8 LD HL,SP+xx",
         "F9 LD SP,HL",
         "FA bb aa LD A,($aabb)",
         "FB EI",
         "FC NOP",
         "FD NOP",
         "FE xx CP $xx",
         "FF RST $38"
      };
      return temp;
   }

   /**
      Returns an array of strings corresponding to CB opcodes

      @return the CB opcode list as a string
   **/

   public String[] createCBOpCodeList()
   {
      String[] temp=
      {
         "CB 00 RLC B",
         "CB 01 RLC C",
         "CB 02 RLC D",
         "CB 03 RLC E",
         "CB 04 RLC H",
         "CB 05 RLC L",
         "CB 06 RLC (HL)",
         "CB 07 RLC A",
         "CB 08 RRC B",
         "CB 09 RRC C",
         "CB 0A RRC D",
         "CB 0B RRC E",
         "CB 0C RRC H",
         "CB 0D RRC L",
         "CB 0E RRC (HL)",
         "CB 0F RRC A",

         "CB 10 RL B",
         "CB 11 RL C",
         "CB 12 RL D",
         "CB 13 RL E",
         "CB 14 RL H",
         "CB 15 RL L",
         "CB 16 RL (HL)",
         "CB 17 RL A",
         "CB 18 RR B",
         "CB 19 RR C",
         "CB 1A RR D",
         "CB 1B RR E",
         "CB 1C RR H",
         "CB 1D RR L",
         "CB 1E RR (HL)",
         "CB 1F RR A",

         "CB 20 SLA B",
         "CB 21 SLA C",
         "CB 22 SLA D",
         "CB 23 SLA E",
         "CB 24 SLA H",
         "CB 25 SLA L",
         "CB 26 SLA (HL)",
         "CB 27 SLA A",
         "CB 28 SRA B",
         "CB 29 SRA C",
         "CB 2A SRA D",
         "CB 2B SRA E",
         "CB 2C SRA H",
         "CB 2D SRA L",
         "CB 2E SRA (HL)",
         "CB 2F SRA A",

         "CB 30 NOP",
         "CB 31 NOP",
         "CB 32 NOP",
         "CB 33 NOP",
         "CB 34 NOP",
         "CB 35 NOP",
         "CB 36 NOP",
         "CB 37 SWAP A",
         "CB 38 SRL B",
         "CB 39 SRL C",
         "CB 3A SRL D",
         "CB 3B SRL E",
         "CB 3C SRL H",
         "CB 3D SRL L",
         "CB 3E SRL (HL)",
         "CB 3F SRL A",

         "CB 40 BIT 0,B",
         "CB 41 BIT 0,C",
         "CB 42 BIT 0,D",
         "CB 43 BIT 0,E",
         "CB 44 BIT 0,H",
         "CB 45 BIT 0,L",
         "CB 46 BIT 0,(HL)",
         "CB 47 BIT 0,A",
         "CB 48 BIT 1,B",
         "CB 49 BIT 1,C",
         "CB 4A BIT 1,D",
         "CB 4B BIT 1,E",
         "CB 4C BIT 1,H",
         "CB 4D BIT 1,L",
         "CB 4E BIT 1,(HL)",
         "CB 4F BIT 1,A",

         "CB 50 BIT 2,B",
         "CB 51 BIT 2,C",
         "CB 52 BIT 2,D",
         "CB 53 BIT 2,E",
         "CB 54 BIT 2,H",
         "CB 55 BIT 2,L",
         "CB 56 BIT 2,(HL)",
         "CB 57 BIT 2,A",
         "CB 58 BIT 3,B",
         "CB 59 BIT 3,C",
         "CB 5A BIT 3,D",
         "CB 5B BIT 3,E",
         "CB 5C BIT 3,H",
         "CB 5D BIT 3,L",
         "CB 5E BIT 3,(HL)",
         "CB 5F BIT 3,A",

         "CB 60 BIT 4,B",
         "CB 61 BIT 4,C",
         "CB 62 BIT 4,D",
         "CB 63 BIT 4,E",
         "CB 64 BIT 4,H",
         "CB 65 BIT 4,L",
         "CB 66 BIT 4,(HL)",
         "CB 67 BIT 4,A",
         "CB 68 BIT 5,B",
         "CB 69 BIT 5,C",
         "CB 6A BIT 5,D",
         "CB 6B BIT 5,E",
         "CB 6C BIT 5,H",
         "CB 6D BIT 5,L",
         "CB 6E BIT 5,(HL)",
         "CB 6F BIT 5,A",

         "CB 70 BIT 6,B",
         "CB 71 BIT 6,C",
         "CB 72 BIT 6,D",
         "CB 73 BIT 6,E",
         "CB 74 BIT 6,H",
         "CB 75 BIT 6,L",
         "CB 76 BIT 6,(HL)",
         "CB 77 BIT 6,A",
         "CB 78 BIT 7,B",
         "CB 79 BIT 7,C",
         "CB 7A BIT 7,D",
         "CB 7B BIT 7,E",
         "CB 7C BIT 7,H",
         "CB 7D BIT 7,L",
         "CB 7E BIT 7,(HL)",
         "CB 7F BIT 7,A",

         "CB 80 RES 0,B",
         "CB 81 RES 0,C",
         "CB 82 RES 0,D",
         "CB 83 RES 0,E",
         "CB 84 RES 0,H",
         "CB 85 RES 0,L",
         "CB 86 RES 0,(HL)",
         "CB 87 RES 0,A",
         "CB 88 RES 1,B",
         "CB 89 RES 1,C",
         "CB 8A RES 1,D",
         "CB 8B RES 1,E",
         "CB 8C RES 1,H",
         "CB 8D RES 1,L",
         "CB 8E RES 1,(HL)",
         "CB 8F RES 1,A",

         "CB 90 RES 2,B",
         "CB 91 RES 2,C",
         "CB 92 RES 2,D",
         "CB 93 RES 2,E",
         "CB 94 RES 2,H",
         "CB 95 RES 2,L",
         "CB 96 RES 2,(HL)",
         "CB 97 RES 2,A",
         "CB 98 RES 3,B",
         "CB 99 RES 3,C",
         "CB 9A RES 3,D",
         "CB 9B RES 3,E",
         "CB 9C RES 3,H",
         "CB 9D RES 3,L",
         "CB 9E RES 3,(HL)",
         "CB 9F RES 3,A",

         "CB A0 RES 4,B",
         "CB A1 RES 4,C",
         "CB A2 RES 4,D",
         "CB A3 RES 4,E",
         "CB A4 RES 4,H",
         "CB A5 RES 4,L",
         "CB A6 RES 4,(HL)",
         "CB A7 RES 4,A",
         "CB A8 RES 5,B",
         "CB A9 RES 5,C",
         "CB AA RES 5,D",
         "CB AB RES 5,E",
         "CB AC RES 5,H",
         "CB AD RES 5,L",
         "CB AE RES 5,(HL)",
         "CB AF RES 5,A",

         "CB B0 RES 6,B",
         "CB B1 RES 6,C",
         "CB B2 RES 6,D",
         "CB B3 RES 6,E",
         "CB B4 RES 6,H",
         "CB B5 RES 6,L",
         "CB B6 RES 6,(HL)",
         "CB B7 RES 6,A",
         "CB B8 RES 7,B",
         "CB B9 RES 7,C",
         "CB BA RES 7,D",
         "CB BB RES 7,E",
         "CB BC RES 7,H",
         "CB BD RES 7,L",
         "CB BE RES 7,(HL)",
         "CB BF RES 7,A",

         "CB C0 SET 0,B",
         "CB C1 SET 0,C",
         "CB C2 SET 0,D",
         "CB C3 SET 0,E",
         "CB C4 SET 0,H",
         "CB C5 SET 0,L",
         "CB C6 SET 0,(HL)",
         "CB C7 SET 0,A",
         "CB C8 SET 1,B",
         "CB C9 SET 1,C",
         "CB CA SET 1,D",
         "CB CB SET 1,E",
         "CB CC SET 1,H",
         "CB CD SET 1,L",
         "CB CE SET 1,(HL)",
         "CB CF SET 1,A",

         "CB D0 SET 2,B",
         "CB D1 SET 2,C",
         "CB D2 SET 2,D",
         "CB D3 SET 2,E",
         "CB D4 SET 2,H",
         "CB D5 SET 2,L",
         "CB D6 SET 2,(HL)",
         "CB D7 SET 2,A",
         "CB D8 SET 3,B",
         "CB D9 SET 3,C",
         "CB DA SET 3,D",
         "CB DB SET 3,E",
         "CB DC SET 3,H",
         "CB DD SET 3,L",
         "CB DE SET 3,(HL)",
         "CB DF SET 3,A",

         "CB E0 SET 4,B",
         "CB E1 SET 4,C",
         "CB E2 SET 4,D",
         "CB E3 SET 4,E",
         "CB E4 SET 4,H",
         "CB E5 SET 4,L",
         "CB E6 SET 4,(HL)",
         "CB E7 SET 4,A",
         "CB E8 SET 5,B",
         "CB E9 SET 5,C",
         "CB EA SET 5,D",
         "CB EB SET 5,E",
         "CB EC SET 5,H",
         "CB ED SET 5,L",
         "CB EE SET 5,(HL)",
         "CB EF SET 5,A",

         "CB F0 SET 6,B",
         "CB F1 SET 6,C",
         "CB F2 SET 6,D",
         "CB F3 SET 6,E",
         "CB F4 SET 6,H",
         "CB F5 SET 6,L",
         "CB F6 SET 6,(HL)",
         "CB F7 SET 6,A",
         "CB F8 SET 7,B",
         "CB F9 SET 7,C",
         "CB FA SET 7,D",
         "CB FB SET 7,E",
         "CB FC SET 7,H",
         "CB FD SET 7,L",
         "CB FE SET 7,(HL)",
         "CB FF SET 7,A"
      };
      return temp;
   }
}