import java.awt.image.*;
import java.awt.*;
import java.util.*;

/**
   Fast Image producing class

   @author David Winchurch.
   @version 09/03/2001
**/

class DirectImage implements ImageProducer
{
   //array to store the raw pixel data in
   public int[] pixels;

   //globaly defined width and height
   public int width, height;

   //Standard java vm colour model
   private ColorModel model=ColorModel.getRGBdefault();

   //vector to keep track of the consumers of the produced image
   private Vector consumers=new Vector();

   /**
      Constructor. Creates and instantiates all the global variables

      @param w the width of the DirectImage
      @param h the height of the DirectImage
   **/

   DirectImage(int w, int h)
   {
      width=w;
      height=h;
      
      //creates an array with enough pixels in it
      pixels=new int[width*height];
   }

   /**
      This method allows for a number of image consumers to have access to
      the image produced from the pixel data by adding them to a Vector
      using this method

      @param ic the ImageConsumer that requires access to this image
   **/

   public synchronized void addConsumer(ImageConsumer ic)
   {
      //checks to see if the ImageConsumer is already in the Vector
      if(!isConsumer(ic))
      {
         consumers.addElement(ic);

         //sets the ImageConsumers attributes so that they are the same as this
         ic.setHints(ImageConsumer.TOPDOWNLEFTRIGHT | ImageConsumer.SINGLEPASS);
         ic.setDimensions(width, height);
         ic.setProperties(new Hashtable());
         ic.setColorModel(model);
      }
   }

   /**
      Checks to see if an ImageConsumer is contained in the consumers Vector

      @param ic the ImageConsumer used for the comparison
      @return true if the specified ImageConsumer is registered by this class
   **/

   public synchronized boolean isConsumer(ImageConsumer ic)
   {
      return consumers.contains(ic);
   }

   /**
      Removes a specified ImageConsumer from the consumer Vector

      @param ic the ImageConsumer to be removed
   **/

   public synchronized void removeConsumer(ImageConsumer ic)
   {
      consumers.removeElement(ic);
   }

   /**
      This method constructs the image and passes it to each of the consumers

      @param ic the ImageConsumer to be registered
   **/

   public void startProduction(ImageConsumer ic)
   {
      addConsumer(ic);

      //An object that will cycle through elements in a vector when nextElement is called
      Enumeration e=consumers.elements();

      while (e.hasMoreElements())
      {
         ImageConsumer icl=(ImageConsumer)e.nextElement();

         //refreshes the pixels in each of the image consumers
         icl.setPixels(0, 0, width, height, model, pixels, 0, width);

         //tells the ImageConsumer that a frame has been refreshed
         icl.imageComplete(ImageConsumer.SINGLEFRAMEDONE);
      }
   }

   /**
      Requests a different method for sending the image data. Used for some
      of the more advanced filters which I dont use so I havent put the method
      in

      @param ic the ImageConsumer to be updated
   **/

   public void requestTopDownLeftRightResend(ImageConsumer ic) {}
}

