import java.awt.*;
import java.awt.event.*;

/**
   A class used to alter the 4 colours used in black and white gameboy emulation

   @author David Winchurch
   @version 10/03/2001
**/

public class PaletteEditor extends Dialog implements ActionListener, AdjustmentListener, ItemListener
{
   //Stores a pointer to the virtual CPU
   private GameBoyCPU CPU;

   //Arrays to store the various color values
   private Color[][] BGPALETTE, ORIGINALBGPALETTE=new Color[2][2],
                     OBJ0PALETTE, ORIGINALOBJ0PALETTE=new Color[2][2],
                     OBJ1PALETTE, ORIGINALOBJ1PALETTE=new Color[2][2],
                     DEFAULTS={{Color.white, Color.lightGray},{Color.gray, Color.black}};

   //The panels required to get the layout right
   private Panel northPanel=new Panel(new GridLayout(6, 1, 3, 3)),
                 bgPalettePanel=new Panel(new GridLayout(1, 4, 3, 3)),
                 obj0PalettePanel=new Panel(new GridLayout(1, 4, 3, 3)),
                 obj1PalettePanel=new Panel(new GridLayout(1, 4, 3, 3)),
                 bgRadioButtonPanel=new Panel(new GridLayout(1, 4, 3, 3)),
                 obj0RadioButtonPanel=new Panel(new GridLayout(1, 4, 3, 3)),
                 obj1RadioButtonPanel=new Panel(new GridLayout(1, 4, 3, 3)),
                 sliderLabelPanel=new Panel(new GridLayout(3, 1)),
                 sliderPanel=new Panel(new GridLayout(3, 1)),
                 buttonPanel=new Panel(new GridLayout(1, 2, 3, 3));

   //The labels required to display the colours
   private Label bgPal0Label=new Label(), bgPal1Label=new Label(),
                 bgPal2Label=new Label(), bgPal3Label=new Label(),
                 obj0Pal0Label=new Label(), obj0Pal1Label=new Label(),
                 obj0Pal2Label=new Label(), obj0Pal3Label=new Label(),
                 obj1Pal0Label=new Label(), obj1Pal1Label=new Label(),
                 obj1Pal2Label=new Label(), obj1Pal3Label=new Label();

   //Scrollbars used to specify the colour for a panel
   private Scrollbar red=new Scrollbar(Scrollbar.HORIZONTAL, 0, 1, 0, 255),
                     green=new Scrollbar(Scrollbar.HORIZONTAL, 0, 1, 0, 255),
                     blue=new Scrollbar(Scrollbar.HORIZONTAL, 0, 1, 0, 255);

   //Buttons
   private Button ok=new Button("OK"),
                  proof=new Button("Proof"),
                  defaults=new Button("Defaults"),
                  cancel=new Button("Cancel");

   //Object that only allows one checkbox to be selected at once
   private CheckboxGroup paletteCheckBoxes=new CheckboxGroup();

   //The checkboxes used to select which color to alter
   private Checkbox bgPal0Check=new Checkbox("BG 0", paletteCheckBoxes, true),
                    bgPal1Check=new Checkbox("BG 1", paletteCheckBoxes, false),
                    bgPal2Check=new Checkbox("BG 2", paletteCheckBoxes, false),
                    bgPal3Check=new Checkbox("BG 3", paletteCheckBoxes, false),
                    obj0Pal0Check=new Checkbox("Obj0 0", paletteCheckBoxes, false),
                    obj0Pal1Check=new Checkbox("Obj0 1", paletteCheckBoxes, false),
                    obj0Pal2Check=new Checkbox("Obj0 2", paletteCheckBoxes, false),
                    obj0Pal3Check=new Checkbox("Obj0 3", paletteCheckBoxes, false),
                    obj1Pal0Check=new Checkbox("Obj1 0", paletteCheckBoxes, false),
                    obj1Pal1Check=new Checkbox("Obj1 1", paletteCheckBoxes, false),
                    obj1Pal2Check=new Checkbox("Obj1 2", paletteCheckBoxes, false),
                    obj1Pal3Check=new Checkbox("Obj1 3", paletteCheckBoxes, false);

   /**
      Constructor

      @param f the parent frame
      @param pal the palette information that requires editing
   **/

   public PaletteEditor(Frame f, GameBoyCPU cpu)
   {
      //calls the DialogBox constuctor to set the parent, title and modality
      super(f, "Palette Editor", true);

      //Sets up the pointers to the relevant objects
      CPU=cpu;
      BGPALETTE=cpu.getBGPalette();
      OBJ0PALETTE=cpu.getOBJ0Palette();
      OBJ1PALETTE=cpu.getOBJ1Palette();

      //sets up the debug window
      setLocation(f.getLocation().x+f.getSize().width ,f.getLocation().y);
      setLayout(new BorderLayout());
      setBackground(Color.lightGray);

      //sets the default values for undo perposes
      ORIGINALBGPALETTE[0][0]=cpu.getBGPalette()[0][0];
      ORIGINALBGPALETTE[0][1]=cpu.getBGPalette()[0][1];
      ORIGINALBGPALETTE[1][0]=cpu.getBGPalette()[1][0];
      ORIGINALBGPALETTE[1][1]=cpu.getBGPalette()[1][1];
      ORIGINALOBJ0PALETTE[0][0]=cpu.getOBJ0Palette()[0][0];
      ORIGINALOBJ0PALETTE[0][1]=cpu.getOBJ0Palette()[0][1];
      ORIGINALOBJ0PALETTE[1][0]=cpu.getOBJ0Palette()[1][0];
      ORIGINALOBJ0PALETTE[1][1]=cpu.getOBJ0Palette()[1][1];
      ORIGINALOBJ1PALETTE[0][0]=cpu.getOBJ1Palette()[0][0];
      ORIGINALOBJ1PALETTE[0][1]=cpu.getOBJ1Palette()[0][1];
      ORIGINALOBJ1PALETTE[1][0]=cpu.getOBJ1Palette()[1][0];
      ORIGINALOBJ1PALETTE[1][1]=cpu.getOBJ1Palette()[1][1];

      //Sets up the labels with their initial color values
      bgPal0Label.setBackground(BGPALETTE[0][0]);
      bgPal1Label.setBackground(BGPALETTE[0][1]);
      bgPal2Label.setBackground(BGPALETTE[1][0]);
      bgPal3Label.setBackground(BGPALETTE[1][1]);
      obj0Pal0Label.setBackground(OBJ0PALETTE[0][0]);
      obj0Pal1Label.setBackground(OBJ0PALETTE[0][1]);
      obj0Pal2Label.setBackground(OBJ0PALETTE[1][0]);
      obj0Pal3Label.setBackground(OBJ0PALETTE[1][1]);
      obj1Pal0Label.setBackground(OBJ1PALETTE[0][0]);
      obj1Pal1Label.setBackground(OBJ1PALETTE[0][1]);
      obj1Pal2Label.setBackground(OBJ1PALETTE[1][0]);
      obj1Pal3Label.setBackground(OBJ1PALETTE[1][1]);

      //Sets the starting values for the slider
      setSliders(bgPal0Label);

      //Adds adjustment listeners to the sliders
      red.addAdjustmentListener(this);
      green.addAdjustmentListener(this);
      blue.addAdjustmentListener(this);

      //Adds action listeners
      bgPal0Check.addItemListener(this);
      bgPal1Check.addItemListener(this);
      bgPal2Check.addItemListener(this);
      bgPal3Check.addItemListener(this);
      obj0Pal0Check.addItemListener(this);
      obj0Pal1Check.addItemListener(this);
      obj0Pal2Check.addItemListener(this);
      obj0Pal3Check.addItemListener(this);
      obj1Pal0Check.addItemListener(this);
      obj1Pal1Check.addItemListener(this);
      obj1Pal2Check.addItemListener(this);
      obj1Pal3Check.addItemListener(this);
      ok.addActionListener(this);
      proof.addActionListener(this);
      defaults.addActionListener(this);
      cancel.addActionListener(this);

      //Adds the labels to their releveant panels
      bgPalettePanel.add(bgPal0Label);
      bgPalettePanel.add(bgPal1Label);
      bgPalettePanel.add(bgPal2Label);
      bgPalettePanel.add(bgPal3Label);
      obj0PalettePanel.add(obj0Pal0Label);
      obj0PalettePanel.add(obj0Pal1Label);
      obj0PalettePanel.add(obj0Pal2Label);
      obj0PalettePanel.add(obj0Pal3Label);
      obj1PalettePanel.add(obj1Pal0Label);
      obj1PalettePanel.add(obj1Pal1Label);
      obj1PalettePanel.add(obj1Pal2Label);
      obj1PalettePanel.add(obj1Pal3Label);

      //Adds the radio buttons to their releveant panels
      bgRadioButtonPanel.add(bgPal0Check);
      bgRadioButtonPanel.add(bgPal1Check);
      bgRadioButtonPanel.add(bgPal2Check);
      bgRadioButtonPanel.add(bgPal3Check);
      obj0RadioButtonPanel.add(obj0Pal0Check);
      obj0RadioButtonPanel.add(obj0Pal1Check);
      obj0RadioButtonPanel.add(obj0Pal2Check);
      obj0RadioButtonPanel.add(obj0Pal3Check);
      obj1RadioButtonPanel.add(obj1Pal0Check);
      obj1RadioButtonPanel.add(obj1Pal1Check);
      obj1RadioButtonPanel.add(obj1Pal2Check);
      obj1RadioButtonPanel.add(obj1Pal3Check);

      //Sets up the sliderLabelPanel
      sliderLabelPanel.add(new Label("Red"));
      sliderLabelPanel.add(new Label("Green"));
      sliderLabelPanel.add(new Label("Blue"));

      //Sets the labels to the relevant colors
      sliderLabelPanel.getComponent(0).setBackground(new Color(255, 200, 200));
      sliderLabelPanel.getComponent(1).setBackground(new Color(200, 255, 200));
      sliderLabelPanel.getComponent(2).setBackground(new Color(200, 200, 255));

      //Sets up the sliderPanel
      sliderPanel.add(red);
      sliderPanel.add(green);
      sliderPanel.add(blue);

      //Sets up the buttonPanel
      buttonPanel.setBackground(new Color(174, 168, 217));
      buttonPanel.add(ok);
      buttonPanel.add(proof);
      buttonPanel.add(defaults);
      buttonPanel.add(cancel);

      //Adds the relevant panels to the north panel
      northPanel.setBackground(new Color(174, 168, 217));
      northPanel.add(bgPalettePanel);
      northPanel.add(bgRadioButtonPanel);
      northPanel.add(obj0PalettePanel);
      northPanel.add(obj0RadioButtonPanel);
      northPanel.add(obj1PalettePanel);
      northPanel.add(obj1RadioButtonPanel);

      //Add the components to their overal position in the layout
      add(northPanel, BorderLayout.NORTH);
      add(sliderLabelPanel, BorderLayout.WEST);
      add(sliderPanel, BorderLayout.CENTER);
      add(buttonPanel, BorderLayout.SOUTH);

      //resizes the window to its optimum size
      pack();

      //Makes this visible
      setVisible(true);
   }

   /**
      Sets the sliders position and color to the correct value for the background
      color of the passed label

      @param l the label whos background color you wan to use
   **/

   public void setSliders(Label l)
   {
      red.setValue(l.getBackground().getRed());
      red.setBackground(new Color(red.getValue(), 0, 0));
      green.setValue(l.getBackground().getGreen());
      green.setBackground(new Color(0, green.getValue(), 0));
      blue.setValue(l.getBackground().getBlue());
      blue.setBackground(new Color(0, 0, blue.getValue()));
   }

   /**
      This is the ActionEvent handler for PaletteEditor. It accepts events generated
      by any component with an ActionListener added.

      @param event The event that was generated.
   **/

   public void actionPerformed(ActionEvent e)
   {
      Object source=e.getSource();

      if(source==ok)
      {
         //Sets all the paletts up with there new values
         BGPALETTE[0][0]=bgPal0Label.getBackground();
         BGPALETTE[0][1]=bgPal1Label.getBackground();
         BGPALETTE[1][0]=bgPal2Label.getBackground();
         BGPALETTE[1][1]=bgPal3Label.getBackground();
         OBJ0PALETTE[0][0]=obj0Pal0Label.getBackground();
         OBJ0PALETTE[0][1]=obj0Pal1Label.getBackground();
         OBJ0PALETTE[1][0]=obj0Pal2Label.getBackground();
         OBJ0PALETTE[1][1]=obj0Pal3Label.getBackground();
         OBJ1PALETTE[0][0]=obj1Pal0Label.getBackground();
         OBJ1PALETTE[0][1]=obj1Pal1Label.getBackground();
         OBJ1PALETTE[1][0]=obj1Pal2Label.getBackground();
         OBJ1PALETTE[1][1]=obj1Pal3Label.getBackground();

         //forces the rom to recalculate the color values its using
         CPU.setBackPalette();
         CPU.setForePalette0();
         CPU.setForePalette1();

         //destroys this
         this.dispose();
      }
      if(source==proof)
      {
         //Sets all the paletts up with there new values
         BGPALETTE[0][0]=bgPal0Label.getBackground();
         BGPALETTE[0][1]=bgPal1Label.getBackground();
         BGPALETTE[1][0]=bgPal2Label.getBackground();
         BGPALETTE[1][1]=bgPal3Label.getBackground();
         OBJ0PALETTE[0][0]=obj0Pal0Label.getBackground();
         OBJ0PALETTE[0][1]=obj0Pal1Label.getBackground();
         OBJ0PALETTE[1][0]=obj0Pal2Label.getBackground();
         OBJ0PALETTE[1][1]=obj0Pal3Label.getBackground();
         OBJ1PALETTE[0][0]=obj1Pal0Label.getBackground();
         OBJ1PALETTE[0][1]=obj1Pal1Label.getBackground();
         OBJ1PALETTE[1][0]=obj1Pal2Label.getBackground();
         OBJ1PALETTE[1][1]=obj1Pal3Label.getBackground();

         //forces the rom to recalculate the color values its using
         CPU.setBackPalette();
         CPU.setForePalette0();
         CPU.setForePalette1();
      }
      if(source==defaults)
      {
         //Sets all the labels back to their default values
         bgPal0Label.setBackground(DEFAULTS[0][0]);
         bgPal1Label.setBackground(DEFAULTS[0][1]);
         bgPal2Label.setBackground(DEFAULTS[1][0]);
         bgPal3Label.setBackground(DEFAULTS[1][1]);
         obj0Pal0Label.setBackground(DEFAULTS[0][0]);
         obj0Pal1Label.setBackground(DEFAULTS[0][1]);
         obj0Pal2Label.setBackground(DEFAULTS[1][0]);
         obj0Pal3Label.setBackground(DEFAULTS[1][1]);
         obj1Pal0Label.setBackground(DEFAULTS[0][0]);
         obj1Pal1Label.setBackground(DEFAULTS[0][1]);
         obj1Pal2Label.setBackground(DEFAULTS[1][0]);
         obj1Pal3Label.setBackground(DEFAULTS[1][1]);

         //sets up the sliders
         if(bgPal0Check.getState()) setSliders(bgPal0Label);
         if(bgPal1Check.getState()) setSliders(bgPal1Label);
         if(bgPal2Check.getState()) setSliders(bgPal2Label);
         if(bgPal3Check.getState()) setSliders(bgPal3Label);
         if(obj0Pal0Check.getState()) setSliders(obj0Pal0Label);
         if(obj0Pal1Check.getState()) setSliders(obj0Pal1Label);
         if(obj0Pal2Check.getState()) setSliders(obj0Pal2Label);
         if(obj0Pal3Check.getState()) setSliders(obj0Pal3Label);
         if(obj1Pal0Check.getState()) setSliders(obj1Pal0Label);
         if(obj1Pal1Check.getState()) setSliders(obj1Pal1Label);
         if(obj1Pal2Check.getState()) setSliders(obj1Pal2Label);
         if(obj1Pal3Check.getState()) setSliders(obj1Pal3Label);
      }
      if(source==cancel)
      {
         //resets the palettes back to their original values
         BGPALETTE[0][0]=ORIGINALBGPALETTE[0][0];
         BGPALETTE[0][1]=ORIGINALBGPALETTE[0][1];
         BGPALETTE[1][0]=ORIGINALBGPALETTE[1][0];
         BGPALETTE[1][1]=ORIGINALBGPALETTE[1][1];
         OBJ0PALETTE[0][0]=ORIGINALOBJ0PALETTE[0][0];
         OBJ0PALETTE[0][1]=ORIGINALOBJ0PALETTE[0][1];
         OBJ0PALETTE[1][0]=ORIGINALOBJ0PALETTE[1][0];
         OBJ0PALETTE[1][1]=ORIGINALOBJ0PALETTE[1][1];
         OBJ1PALETTE[0][0]=ORIGINALOBJ1PALETTE[0][0];
         OBJ1PALETTE[0][1]=ORIGINALOBJ1PALETTE[0][1];
         OBJ1PALETTE[1][0]=ORIGINALOBJ1PALETTE[1][0];
         OBJ1PALETTE[1][1]=ORIGINALOBJ1PALETTE[1][1];

         //forces the rom to recalculate the color values its using
         CPU.setBackPalette();
         CPU.setForePalette0();
         CPU.setForePalette1();

         //destroys this
         this.dispose();
      }
   }

   /**
      This is the AdjustmentEvent handler for PaletteEditor. It accepts events
      generated by any component with an AdjustmentListener added.

      @param event The event that was generated.
   **/

   public void adjustmentValueChanged(AdjustmentEvent e)
   {
      Object source=e.getSource();

      //Sets the color of the sliders
      if(source==red) red.setBackground(new Color(red.getValue(), 0, 0));
      if(source==green) green.setBackground(new Color(0, green.getValue(), 0));
      if(source==blue) blue.setBackground(new Color(0, 0, blue.getValue()));

      //Calculates the background color for the relevant label
      if(bgPal0Check.getState()) bgPal0Label.setBackground(new Color(red.getValue(), green.getValue(), blue.getValue()));
      if(bgPal1Check.getState()) bgPal1Label.setBackground(new Color(red.getValue(), green.getValue(), blue.getValue()));
      if(bgPal2Check.getState()) bgPal2Label.setBackground(new Color(red.getValue(), green.getValue(), blue.getValue()));
      if(bgPal3Check.getState()) bgPal3Label.setBackground(new Color(red.getValue(), green.getValue(), blue.getValue()));
      if(obj0Pal0Check.getState()) obj0Pal0Label.setBackground(new Color(red.getValue(), green.getValue(), blue.getValue()));
      if(obj0Pal1Check.getState()) obj0Pal1Label.setBackground(new Color(red.getValue(), green.getValue(), blue.getValue()));
      if(obj0Pal2Check.getState()) obj0Pal2Label.setBackground(new Color(red.getValue(), green.getValue(), blue.getValue()));
      if(obj0Pal3Check.getState()) obj0Pal3Label.setBackground(new Color(red.getValue(), green.getValue(), blue.getValue()));
      if(obj1Pal0Check.getState()) obj1Pal0Label.setBackground(new Color(red.getValue(), green.getValue(), blue.getValue()));
      if(obj1Pal1Check.getState()) obj1Pal1Label.setBackground(new Color(red.getValue(), green.getValue(), blue.getValue()));
      if(obj1Pal2Check.getState()) obj1Pal2Label.setBackground(new Color(red.getValue(), green.getValue(), blue.getValue()));
      if(obj1Pal3Check.getState()) obj1Pal3Label.setBackground(new Color(red.getValue(), green.getValue(), blue.getValue()));
   }

   /**
      This is the ItemEvent handler for PaletteEditor. It accepts events
      generated by any component with an ItemListener added.

      @param event The event that was generated.
   **/

   public void itemStateChanged(ItemEvent e)
   {
      Object source=e.getSource();

      if(source==bgPal0Check) setSliders(bgPal0Label);
      if(source==bgPal1Check) setSliders(bgPal1Label);
      if(source==bgPal2Check) setSliders(bgPal2Label);
      if(source==bgPal3Check) setSliders(bgPal3Label);
      if(source==obj0Pal0Check) setSliders(obj0Pal0Label);
      if(source==obj0Pal1Check) setSliders(obj0Pal1Label);
      if(source==obj0Pal2Check) setSliders(obj0Pal2Label);
      if(source==obj0Pal3Check) setSliders(obj0Pal3Label);
      if(source==obj1Pal0Check) setSliders(obj1Pal0Label);
      if(source==obj1Pal1Check) setSliders(obj1Pal1Label);
      if(source==obj1Pal2Check) setSliders(obj1Pal2Label);
      if(source==obj1Pal3Check) setSliders(obj1Pal3Label);
   }
}